# -*- coding: utf-8 -*-
import os

import xml.dom.minidom
import codecs


class XMLWrapper:
	"""Syntax sugar for xml.dom.minidom:
	__getitem__ calls getElementsByTagName and
	__getattr__ calls getAttribute
	modified from OTableau's pydom (by N. Sanders)
	"""
	
	def __init__(self, instream):
		if isinstance(instream, str):
			instream = unicode(instream, 'utf-8')
		
		if isinstance(instream, unicode):
			buffer = codecs.open(instream, 'r', 'utf-8') 
			xmlString = buffer.read()
			buffer.close()
			self.data = xml.dom.minidom.parseString(xmlString.encode( "utf-8" ))
		else:
			self.data = instream
			
	def __getitem__(self, item):
		return map(XMLWrapper, self.data.getElementsByTagName(item))
		
	def __getattr__(self, attr):
		if attr in dir(self.data):
			return getattr(self.data, attr)
		else:
			return self.data.getAttribute(attr)
			
	def getValue(self, tag):
		try:
			return self[tag][0].firstChild.data
		except AttributeError:
			return ''
			
class XMLBasicWriter:
	def __init__(self, root):
		self.data = [ ]
		self.root = root
		
	def __setitem__(self, item, value):
		self.data.append((item, value))
		
	def __getitem__(self, item):
		for key, value in self.data:
			if key == item:
				return value
				
		raise KeyError
	
	def write(self, outfile):
		out = codecs.open(outfile, encoding='utf-8', mode='w')
		
		out.write(u'<?xml version="1.0" encoding="UTF-8"?>\n\n')
		out.write(u'<%s>\n' % self.root)
		
		for tag, value in self.data:
			try:
				out.write(u'\t<%s>%s</%s>\n' % (tag, unicode(value, "UTF-8"), tag))
			# la force brute...
			except TypeError:
				out.write(u'\t<%s>%s</%s>\n' % (tag, value, tag))
		out.write('</%s>\n' % self.root)
		out.close()


def getTagsFromFile(file, tags):
	"Extract tag:values from a file"
	meta = { }
			
	if os.path.exists(file):
		doc = XMLWrapper(file)
		
		for tag in tags:
			meta[tag] = doc.getValue(tag)
			
	return meta
	
	
if __name__ == '__main__':
	doc = XMLWrapper('preferences.xml')
	print doc # -> '11a'
#	doc = XMLVeryBasicWriter('PFCSpeaker')
#	doc['ID'] = 'TG1'
#	doc['Description'] = 'Innovateur'
#	doc.write('test.pfcmeta')