# -*- coding: utf-8 -*-
import sys, os
import wx
import codecs

import update
from core.xmlwrap import XMLBasicWriter as XMLWriter
from core.open import startfile
from core.zip import zipfolder
# GUI
from messages import *
from editors import editInterval
# PFC
from pfc.routines import *
from pfc.meta import MetadataTable
# praat
from praat import praat


def OnAddSurveyToCorpus(parent):
    survey = None
    dialog = wx.DirDialog(None, u"Choisir l'enquête à ajouter", 
                  style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON) 
    
    if dialog.ShowModal() == wx.ID_OK: 
        survey = dialog.GetPath()
    dialog.Destroy() 
 
    if not survey:
        return
    
    createSurveyAlias(survey, parent.corpus.path)
    info(u"Vous devez relancer la plateforme pour que les changements prennent effet")
        

        
def OnSerializeCorpus(parent):
    try:
        from pfc import serialize
    except ImportError:
        alert(u"La sérialisation n'est pas disponible sur votre ordinateur.", "Module pickle absent"); return
    
    bar = wx.ProgressDialog(u"Enregistrement en cours", u"Veuillez patienter", 1, 
                     style = wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME)

    serialize.dump(parent.corpus, parent.serializedPath) 
    bar.Update(1)
    bar.Destroy()    

def OnWriteCorpusMetadata(parent):
    editor = parent.brows['meta']
    name  = editor['Name'].GetValue()
    date  = editor['Date'].GetValue()
    version  = editor['Version'].GetValue()
    desc = editor['Description'].GetValue()
        
    parent.corpus.meta = { } # blank
    doc = XMLWriter('PFCSurvey')
    
    for tag, value in [('Name', name), ('Date', date), ('Version', version), ('Description', desc)]:
        doc[tag] = value
        parent.corpus.meta[tag] = value
    
    doc.write(os.path.join(parent.corpus.path, 'corpus-metadata.xml'))    
    info(u"Les métadonnées du corpus ont été enregistrées. Relancez la plateforme pour que les changements prennent effet.")

    
def OnWriteSurveyMetadata(parent):
    editor = parent.brows['meta']    
    srv  = editor['Survey'].GetLabel().lower()
    date  = editor['Date'].GetValue()
    version  = editor['Version'].GetValue()
    desc = editor['Description'].GetValue()
        
    parent[srv].meta = { } # blank
    doc = XMLWriter('PFCSurvey')
    
    for tag, value in [('ID', srv), ('Date', date), ('Version', version), ('Description', desc)]:
        doc[tag] = value
        parent[srv].meta[tag] = value
    
    doc.write(os.path.join(parent[srv].path, srv + '-metadata.xml'))
    
    info(u"Les métadonnées de l'enquête %s ont été enregistrées. Relancez la plateforme pour que les changements prennent effet." % srv.upper())
    
    
def OnWriteSpeakerMetadata(parent):
    editor = parent.brows['meta']    
    spkr = editor['Speaker'].GetLabel().lower()
    srv  = editor['Survey'].GetLabel().lower()
    age  = editor['Age'].GetValue()
    job  = editor['Job'].GetValue()
    desc = editor['Description'].GetValue()

    sexID  = editor['Sex'].GetSelection()
    sex = editor['Sex'].GetString(sexID)    
    
    parent[srv][spkr].meta = { } # blank in-memory metadata    
    doc = XMLWriter('PFCSpeaker')
    
    for tag, value in [('ID', spkr), ('Survey', srv), ('Age', age), ('Sex', sex), ('Job', job), ('Description', desc)]:
        doc[tag]  = value
        parent[srv][spkr].meta[tag] = value
    
    datadir = os.path.join(parent[srv][spkr].path, 'data')
    metadir = os.path.join(datadir, 'metadata')
    outfile = os.path.join(metadir, srv+spkr+'-metadata.xml')
    if not os.path.exists(datadir):
        os.mkdir(datadir)
    if not os.path.exists(metadir):
        os.mkdir(metadir)
    doc.write(outfile)
        
    info(u"Les métadonnées de %s ont été enregistrées. Relancez la plateforme pour que les changements prennent effet." % spkr.upper())


def OnWriteTaskMetadata(parent):
    editor = parent.brows['meta']    
    spkr = editor['Speaker'].GetLabel().lower()
    srv  = editor['Survey'].GetLabel().lower()
    tsk  = reverseLongTaskNames[editor['Task'].GetLabel()]
    desc = editor['Description'].GetValue()

    parent[srv][spkr].meta = { } # blank in-memory metadata    
    doc = XMLWriter('PFCTask')
    
    for tag, value in [('ID', tsk), ('Survey', srv), ('Speaker', spkr), ('Description', desc)]:
        doc[tag]  = value
        parent[srv][spkr].meta[tag] = value
    
    datadir = os.path.join(parent[srv][spkr].path, 'data')
    metadir = os.path.join(datadir, 'metadata')
    outfile = os.path.join(metadir, srv+spkr+tsk+'-metadata.xml')
    if not os.path.exists(datadir):
        os.mkdir(datadir)
    if not os.path.exists(metadir):
        os.mkdir(metadir)
    doc.write(outfile)

    info(u"Les métadonnées de la tâche \"%s\" ont été enregistrées. Relancez la plateforme pour que les changements prennent effet." % longTaskNames[tsk])
    
    
def OnImportMetadata(parent):
 	"Import metadata from file 'locuteurs.csv'"
 
 	dialog = wx.FileDialog (None, u"Ouvrez le fichier locuteurs...", wildcard = "CSV (*.csv)|*.csv")  
	if dialog.ShowModal() == wx.ID_OK: 
		path = dialog.GetPath()
	dialog.Destroy() 
	
	if not path:
		return
		
	metatable = MetadataTable(path)
 	
 	parent = parent.corpus
 	for srv in parent.keys():
 		for spkr in parent[srv].keys():
 			data = metatable[srv + spkr]

 			if data == {}:
 				continue
 			
 			doc = XMLWriter('PFCSpeaker')	
		
 			# write metadata in the Description field. 
 			desc = ""
 			job = ''
 			age_guessed = False
 			for tag in data.keys():
 				# there are some bugs in the locuteurs.csv file
 				value = data[tag]
 				if tag == 'codelocuteur' or tag == 'nom' or tag == 'prenom' or tag == 'age':
 					pass
					
 				elif tag == 'sexe':
 					if value.lower().startswith('f'):
 						sex = 'Femme'
 					elif value.lower().startswith('h') or value.lower().startswith('m'):
 						sex = 'Homme'
 					# this should not happen
 					else:
 						sex = ''
  				elif tag == 'professions_anciennes':
 					job = value	
 										
 				elif value != 'NULL' and tag != None and value != None: #FIXME: why are there empty labels on windows?
 					desc += tag + ' = ' + value + ', '
 					# to do

 			for tag, value in [('ID', spkr), ('Survey', srv), ('Age', data['age']), ('Sex', sex), ('Job', job), 
							('Description', desc.replace(';', ','))]:
				doc[tag]  = value
			
			datadir = os.path.join(parent[srv][spkr].path, 'data')
			metadir = os.path.join(datadir, 'metadata')
			outfile = os.path.join(metadir, srv+spkr+'-metadata.xml')
			if not os.path.exists(datadir):
				os.mkdir(datadir)
			if not os.path.exists(metadir):
				os.mkdir(metadir)
			doc.write(outfile)

	info(u"Les métadonnées ont été écrites. Relancez la plateforme pour que les changements prennent effet.")


def OnOpen(parent):
    cancel = False
    dialog = wx.DirDialog(None, u'Choisir le corpus', 
                  style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON) 
    if dialog.ShowModal() == wx.ID_OK: 
        parent.root = dialog.GetPath()
    else:
        cancel = True
    dialog.Destroy() 
    
    return cancel        
    
        
def OnDocumentation(e = None):
    doc = os.path.join(os.path.abspath('.'),'documentation', 'documentation_plateforme_pfc.pdf')
    startfile(doc)
    
    
def CheckUpdate(parent, silent = True):
    try:
        msg = update.needsUpdate(parent.prefs['update']['url'])
        if msg:
            info(u"Une nouvelle version est disponible en ligne. %s" % unicode(msg, 'utf-8'), u"Mise à jour")
            
        elif not silent:
            info(u"Aucune mise à jour n'est disponible.", u"Mise à jour")
    except IOError:
        if not silent:
            alert(u"Il semble que vous ne soyez pas connecté à Internet.", u"Erreur")
        else:
            pass

def ZipFolder(e = None):
	cancel = False
	root = ''
	
	dialog = wx.DirDialog(None, u'Choisir le dossier à archiver', 
				  style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON) 
	if dialog.ShowModal() == wx.ID_OK: 
		root = dialog.GetPath()
	else:
		cancel = True
	dialog.Destroy() 
	
	if cancel:
		return
	
	zipname = zipfolder(os.path.dirname(root), root, ['.textgrid', '.intervaltier','.xml'])
	info(u"L'archive %s a été créée." % zipname)
	

def OnEditInterval(parent):
	selected = parent.GetSelectedMatch()
	if not selected:
		alert(u"Sélectionnez un intervalle.")
		return
	tier = parent.tier.GetSelection() + 1
	tgd = selected.task.textgrid
	interval = selected.interval + 1
	buffer = os.path.join(parent.prefsdir, 'label.txt')
	
	getLabel = "Read from file... %s\npath$ = Get label of interval... %d %d\npath$ > %s" % \
		(tgd.path, tier, interval, buffer)

	praat.executeScript(getLabel)

	f = codecs.open(buffer, 'r', "utf-16")
	label = f.read().strip()
	f.close()
	os.remove(buffer)
		
	label = editInterval(tgd.path, tier, interval, label)
	if not label:
		return

	setLabel = "Read from file... %s\nSet interval text... %d %d  %s\nWrite to text file... %s" \
		% (tgd.path, tier, interval, label, tgd.path)
		
	praat.executeScript(setLabel)
		
	

def QuitPlatform():
    sys.exit()
