# -*- coding: utf-8 -*-

'''
The prefs panel allows to (re)set paths for non default installations. This is
particularly useful on Windows.
'''
import sys
import wx
import wx.lib.filebrowsebutton as filebrowse
from pfc import prefs

class BadPlatform(Exception):
	pass



class PrefsPanel(wx.Frame):
	def __init__(self, parent, title = u'Préférences'):
		# windows has a "program folder" entry.
		height = 210
		if sys.platform.startswith('win'):
			height += 35
			
		wx.Frame.__init__(self, parent, -1, title, pos = (50, 50), size = (450, height))#, 
				#style = wx.DEFAULT_FRAME_STYLE ^ (wx.RESIZE_BORDER | wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX))
		panel = wx.Panel(self, -1, size = (450, height))

		self.prefs = parent.prefs.copy() # leave original prefs untouched
		self.prefspath = parent.prefspath
		
		# Root dir
		self.fbroot = filebrowse.DirBrowseButton(panel, -1, size = (420, -1),
				changeCallback = self.OnSelectRoot, buttonText = 'Choisir')
		self.fbroot.SetLabel('Dossier racine')
		self.fbroot.SetValue(self.prefs['path']['corpusdir'])


		# Praat dir
		self.fbpraatdir = filebrowse.DirBrowseButton(panel, -1, size = (420, -1),
				changeCallback = self.OnSelectPraatDir, buttonText = 'Choisir')
		self.fbpraatdir.SetLabel('Dossier Praat')
		
		
		self.fbpraatdir.SetValue(self.GetPraatPath())
		
		# check updates
		self.cbupdate = wx.CheckBox(panel, -1, u"Vérifier les mises à jour au lancement de la plateforme")
		if self.prefs['update']['check'] == 'True':
			self.cbupdate.SetValue(True)
			
		# display warnings for missing files
		self.cbmissing = wx.CheckBox(panel, -1, u"Signaler les fichiers manquants ou mal formatés (schwa et liaison)")
		if self.prefs['application']['catchmissing'] == 'True':
			self.cbmissing.SetValue(True)		
			
		btnOK = wx.Button(panel, -1, "Valider")
		btnCancel = wx.Button(panel, -1, "Annuler")
						
		sizer = wx.BoxSizer(wx.VERTICAL)
		sizer.Add(self.fbroot, 0, wx.ALL, 5)
		sizer.Add(self.fbpraatdir, 0, wx.ALL, 5)

##		# program path on windows
##		if sys.platform.startswith('win'):
##			self.fbwinapp = filebrowse.DirBrowseButton(panel, -1, size = (420, -1),
##				changeCallback = self.OnSelectWinAppDir, buttonText = 'Choisir')
##			self.fbwinapp.SetLabel(u"Dossier programme")
##			self.fbwinapp.SetValue(self.prefs['path']['winappdir'])
##			sizer.Add(self.fbwinapp, 0, wx.ALL, 5)

		sizer.Add(self.cbupdate, 0, wx.ALL, 5)
		sizer.Add(self.cbmissing, 0, wx.ALL, 5)
				
		btns = wx.BoxSizer(wx.HORIZONTAL)
		btns.Add(btnCancel, 0, wx.ALL, 5)
		btns.Add(btnOK, 0, wx.ALL, 5)
		sizer.Add(btns, 0, wx.ALL, 5)
		self.SetSizer(sizer)
		
		self.Bind(wx.EVT_CHECKBOX, self.OnCheckUpdate, self.cbupdate)
		self.Bind(wx.EVT_CHECKBOX, self.OnCatchMissing, self.cbmissing)
		self.Bind(wx.EVT_BUTTON, self.OnValidate, btnOK)
		self.Bind(wx.EVT_BUTTON, self.OnClose, btnCancel)
		self.Bind(wx.EVT_CLOSE, self.OnClose)
		
		self.Show(True)
		
	
	def GetPraatPath(self):
		'Set praat dir in a cross-platform way'
		if sys.platform.startswith('win'):
			return self.prefs['path']['winpraatdir']
		elif sys.platform.startswith('linux'):
			return self.prefs['path']['linuxpraatdir']	
		elif sys.platform == 'darwin':
			return self.prefs['path']['macpraatdir']
		else:
			raise BadPlatform, "Unsupported platform: %s" % sys.platform
		
	def OnSelectRoot(self, e = None):
		val = self.fbroot.GetValue()
		if val:
			self.prefs['path']['corpusdir'] = val

	def OnSelectPraatDir(self, e = None):
		value = self.fbpraatdir.GetValue()
		if not value:
			return

		if sys.platform.startswith('win'):
			self.prefs['path']['winpraatdir'] = value
		elif sys.platform.startswith('linux'):
			self.prefs['path']['linuxpraatdir']	= value
		elif sys.platform == 'darwin':
			self.prefs['path']['macpraatdir'] = value
		else:
			raise BadPlatform, "Unsupported platform: %s" % sys.platform
		
##	def OnSelectWinAppDir(self, e = None):
##		value = self.fbwinapp.GetValue()
##		if value:
##			self.prefs['path']['winappdir'] = value
			
	def OnCheckUpdate(self, e):
		self.prefs['update']['check'] = str(self.cbupdate.IsChecked())

	def OnCatchMissing(self, e):
		self.prefs['application']['catchmissing'] = str(self.cbmissing.IsChecked())		
		
	def OnClose(self, e = None):
		self.Destroy()
		
	def OnValidate(self, e):
		'Update INI file.'
		
		# FIXME: changecallback doen't behave as expected -> refresh manually
		self.OnSelectRoot()
		self.OnSelectPraatDir()
##		if sys.platform.startswith('win'):
##			self.OnSelectWinAppDir()
		
		prefs.writeINI(self.prefspath, self.prefs)
		self.OnClose()
