# meta.py
# this module gathers functions and classes related to metadata processing
import os

class MetadataTable:
    "Store speaker metadata from a CSV file"
    def __init__(self, path):
        self.path = path     
        self.data = [ ]
        self.header = [ ]
        
        self.parse()
        
                     
    def __getitem__(self, speaker):
        'Get metadata for a speaker as a list (speaker is chopped)'
        info = { }

        for ln in self.data:
            if ln[0] == speaker:
            	ln.pop(0)
            	map( (lambda field, value: info.__setitem__(field, value)), self.header, ln )
            	
        return info
        

    def parse(self):
        f = open(self.path)
        self.data = map(lambda ln: ln.split(';'), f.readlines())
        f.close()
        
        self.header = self.data.pop(0)
        self.header.pop(0) # get rid of speaker field
       
        
    def getField(self, speaker, field):
        for i in range(len(self.header)):
            if self.header[i] == field:
                for line in self.data:
                    if line[0] == speaker:
                        return line[i]
                    
        return None
       
       
    def toXml(self):
    	pass
   

def filterSpeakers(browser, speakers, meta, logfile):
	"Filter speakers on the basis of metadata criteria."
	baddies = set() # speaker who don't meet one of the criteria
	bf = { } # final buffer
	metaMissing = set() # speakers who don't have a metadata
	if not meta['values']:
		return speakers
		
	# look at each speaker. If it meets all the conditions, add it to the buffer
	for srv in speakers.keys():
		for spkr in speakers[srv]:
			speaker = browser[srv][spkr]
			sdata = speaker.meta
			srvdata = browser[srv].meta
			spkrId = srv+spkr
			# First age criterion
			if meta['values']['Age']:
				if not 'Age' in sdata or int(sdata['Age']) == 0:
					metaMissing.add(spkrId)
					baddies.add(spkrId) ; continue
				elif not eval(sdata['Age'] \
				+ meta['choices']['Age'] + meta['values']['Age']):
					
					baddies.add(spkrId) ; continue
			# second age criterion
			if meta['values']['Age2']:
				if not 'Age' in sdata or int(sdata['Age']) == 0:
					metaMissing.add(spkrId)
					baddies.add(spkrId) ; continue				
				elif not eval(sdata['Age'] \
				+ meta['choices']['Age2'] + meta['values']['Age2']):
					baddies.add(spkrId) ; continue					
			# Sex
			if meta['choices']['Sex']:
				if not 'Sex' in sdata or not sdata['Sex']:
					metaMissing.add(spkrId)
					baddies.add(spkrId) ; continue
				
				elif sdata['Sex'] != meta['choices']['Sex']:
					baddies.add(spkrId) ; continue

			## The logic is a bit twisted here (Job and Desc.). If users don't have 
			## the key when the search operator is False (defalut case), they are  
			## kept because since they don't have the key, they can't have the tag 
			## in the key.
			
			# Job
			if meta['values']['Job']:
				jobs = meta['values']['Job'].split(';')
				if meta['choices']['Job'] == True:
					if not 'Job' in sdata:
						metaMissing.add(spkrId)
						baddies.add(spkrId) ; continue
						
					elif sdata['Job'] in jobs: 
						baddies.add(srv+spkr) ; continue
					
				elif sdata.has_key('Job') and sdata['Job'] in jobs:
						baddies.add(srv+spkr) ; continue
			# Description (user-specific tags)
			if meta['values']['Description']:
				tags = meta['values']['Description'].split(';')
				if meta['choices']['Description'] == True:
					if not sdata.has_key('Description'):
						baddies.add(srv+spkr) ; continue
					for tag in tags:
						if sdata['Description'].find(tag) == -1:
							baddies.add(srv+spkr)
				elif sdata.has_key('Description'):
					for tag in tags:
						if sdata['Description'].find(tag) > -1:
							baddies.add(srv+spkr); continue
						
			# Extras: survey descriptions 
			if meta['extra']['SurveyDesc']:
				tags = meta['extra']['SurveyDesc'].split(';')
				if meta['choices']['SurveyDesc'] == True:
					if not 'Description' in srvdata:
						baddies.add(srv+spkr); continue
					for tag in tags:
						if srvdata['Description'].find(tag) == -1:
							baddies.add(srv+spkr)
				elif 'Description' in srvdata:
					for tag in tags:
						if srvdata['Description'].find(tag) > -1:
							baddies.add(srv+spkr)

	# append speaker to the buffer
	for srv in speakers:
		for spkr in speakers[srv]:
			if not srv+spkr in baddies:							
				try:
					bf[srv].append(spkr)
				except KeyError:
					bf[srv] = [spkr]
					
	if metaMissing:
		_log(metaMissing, logfile)
	# remove log file is there were no missing metadata
	elif os.path.exists(logfile):
		os.remove(logfile)

	return bf	

def _log(speakers, path):
	"Log missing speakers"
	f = open(path, 'w')
	f.write('\n'.join(speakers))
	f.close()

if __name__ == '__main__':
    table = MetadataTable('/Users/Julien/Desktop/locuteurs8.csv')
    print table['21amb1']
    
    
