import sys, os
#from ConfigParser import ConfigParser
# In version 1.0, preferences were stored in prefs.ini in the INI format.
# Version 2.0 uses XML and stores preferences in preferences.xml
from xml.etree.ElementTree import ElementTree
import codecs


def getPrefsDir():
	prefs = 'PFCPrefs'		
	home = 'HOME'
	if sys.platform.startswith('win'):
		home = 'USERPROFILE'
		
	if sys.platform.startswith('darwin'):
		prefs = os.path.join('Library', 'Preferences', prefs)
	elif sys.platform.startswith('linux'):
		prefs = '.' + prefs

	return os.path.join(os.environ[home], prefs) 
	
	
def getPrefsPath():
	return os.path.join(getPrefsDir(), 'preferences.xml')


def getDefaultINI():
	return \
	{
		'path': 
		{
			'corpusdir': '~/PFC/Corpus',
			'winpraatdir': 'C:\Praat',
			'macpraatdir': '/Applications',
			'linuxpraatdir': '/usr/local/bin',
		},
		'update':
		{
			'check': 'False',
			'url': 'http://www.jeychenne.net/files/platform/version.txt',
		},
		'application':
		{
			'catchmissing': 'True',
		},
	}

def readINI(path, config = { }):
	config = config.copy() 
	
	root = ElementTree(file = path)
	
	for element in root.getiterator():
		value = element.text
		option = element.tag
		try:
			section = element.attrib['class']
		except KeyError:
			continue
			
		if config.has_key(section):
			config[section][option] = value
		else:
			config[section] = {option: value}
	
	return config

	
def writeINI(path, config):
	out = codecs.open(path, encoding='utf-8', mode='w')
	
	out.write(u'<?xml version="1.0" encoding="UTF-8"?>\n\n')
	out.write(u'<preferences>\n')
	
	for section in config.keys():
		for option in config[section].keys():
			value = config[section][option]
			out.write(u"\t<%s class=\"%s\">%s</%s>\n" % (option, section, value, option))
			
	out.write('</preferences>\n')		
	out.close()
	
		
def winHide(file):
	'Set "hidden" attribute for "file".'
	import win32con, win32api
	win32api.SetFileAttributes(file, win32con.FILE_ATTRIBUTE_HIDDEN)


if __name__ == '__main__':
	pass
		
		
		
		
		
		
		
		
